@@ -5,7 +5,7 @@ Term::ReadLine::Tiny - Tiny implementation of ReadLine
 
 =head1 VERSION
 
-version 1.03
+version 1.04
 
 =head1 SYNOPSIS
 
@@ -36,8 +36,6 @@ B<C<Enter> or C<^J> or C<^M>:> Gets input line. Returns the line unless C<EOF> o
 
 B<C<BackSpace> or C<^H> or C<^?>:> Deletes one character behind cursor.
 
-B<C<Delete>:> Deletes one character at cursor. Does nothing if no character at cursor.
-
 B<C<UpArrow>:> Changes line to previous history line.
 
 B<C<DownArrow>:> Changes line to next history line.
@@ -50,6 +48,14 @@ B<C<Home>:> Moves cursor to the start of the line.
 
 B<C<End>:> Moves cursor to the end of the line.
 
+B<C<PageUp>:> Change line to first line of history.
+
+B<C<PageDown>:> Change line to latest line of history.
+
+B<C<Insert>:> Switch typing mode between insert and overwrite.
+
+B<C<Delete>:> Deletes one character at cursor. Does nothing if no character at cursor.
+
 B<C<^D>:> Aborts the operation. Returns C<undef>.
 
 =cut
@@ -58,6 +64,8 @@ use warnings;
 use v5.10.1;
 use feature qw(switch);
 no if ($] >= 5.018), 'warnings' => 'experimental';
+require utf8;
+require PerlIO;
 require Term::ReadLine;
 require Term::ReadKey;
 
@@ -65,16 +73,17 @@ require Term::ReadKey;
 BEGIN
 {
 	require Exporter;
-	our $VERSION     = '1.03';
+	our $VERSION     = '1.04';
 	our @ISA         = qw(Exporter);
 	our @EXPORT      = qw();
 	our @EXPORT_OK   = qw();
 }
 
 
-=head1 Standard Term::ReadLine Methods and Functions
+=head1 Standard Methods and Functions
 
 =cut
+
 =head2 ReadLine()
 
 returns the actual package that executes the commands. If this package is used, the value is C<Term::ReadLine::Tiny>.
@@ -101,8 +110,6 @@ sub new
 	my $self = {};
 	bless $self, $class;
 
-	$self->newTTY($IN, $OUT);
-
 	$self->{readmode} = '';
 	$self->{history} = [];
 
@@ -111,8 +118,12 @@ sub new
 	$self->{features}->{addhistory} = 1;
 	$self->{features}->{minline} = 1;
 	$self->{features}->{autohistory} = 1;
+	$self->{features}->{gethistory} = 1;
+	$self->{features}->{sethistory} = 1;
 	$self->{features}->{changehistory} = 1;
 
+	$self->newTTY($IN, $OUT);
+
 	return $self;
 }
 
@@ -164,7 +175,9 @@ sub readline
 		my $a = substr($line, $index);
 		@line = @line[0..$index-1];
 		$line = substr($line, 0, $index);
+		print $out "\x7F";
 		print $out "\e[J";
+		print $out "\e[D";
 		for my $c (split("", $text))
 		{
 			$s = encode_controlchar($c);
@@ -407,15 +420,28 @@ sub readline
 
 =head2 addhistory($line1[, $line2[, ...]])
 
+B<AddHistory($line1[, $line2[, ...]])>
+
 adds lines to the history of input.
 
 =cut
 sub addhistory
 {
 	my $self = shift;
+	if ($self->{features}->{utf8})
+	{
+		for (my $i = 0; $i < @_; $i++)
+		{
+			utf8::decode($_[$i]);
+		}
+	}
 	push @{$self->{history}}, @_;
 	return (@_);
 }
+sub AddHistory
+{
+	return addhistory(@_);
+}
 
 =head2 IN()
 
@@ -441,6 +467,8 @@ sub OUT
 
 =head2 MinLine([$minline])
 
+B<minline([$minline])>
+
 If argument is specified, it is an advice on minimal size of line to be included into history.
 C<undef> means do not include anything into history (autohistory off).
 
@@ -456,6 +484,10 @@ sub MinLine
 	$self->{features}->{autohistory} = defined($self->{features}->{minline});
 	return $result;
 }
+sub minline
+{
+	return MinLine(@_);
+}
 
 =head2 findConsole()
 
@@ -490,7 +522,7 @@ I<appname> is not present and is the name of the application. B<But not supporte
 
 =item *
 
-I<addhistory> is present, always 1.
+I<addhistory> is present, always C<TRUE>.
 
 =item *
 
@@ -498,12 +530,24 @@ I<minline> is present, default 1. See C<MinLine> method.
 
 =item *
 
-I<autohistory> is present, C<FALSE> if minline is C<undef>. See C<MinLine> method.
+I<autohistory> is present. C<FALSE> if minline is C<undef>. See C<MinLine> method.
+
+=item *
+
+I<gethistory> is present, always C<TRUE>.
+
+=item *
+
+I<sethistory> is present, always C<TRUE>.
 
 =item *
 
 I<changehistory> is present, default C<TRUE>. See C<changehistory> method.
 
+=item *
+
+I<utf8> is present. C<TRUE> if input file handle has C<:utf8> layer.
+
 =back
 
 =cut
@@ -514,9 +558,10 @@ sub Features
 	return \%features;
 }
 
-=head1 Additional Term::ReadLine Methods and Functions
+=head1 Additional Methods and Functions
 
 =cut
+
 =head2 newTTY([$IN[, $OUT]])
 
 takes two arguments which are input filehandle and output filehandle. Switches to use these filehandles.
@@ -528,23 +573,101 @@ sub newTTY
 	my ($IN, $OUT) = @_;
 
 	my ($console, $consoleOUT) = findConsole();
-	my $in = $IN if ref($IN) eq "GLOB";
+	my $console_utf8 = defined($ENV{LANG}) && $ENV{LANG} =~ /\.UTF\-?8$/i;
+	my $console_layers = "";
+	$console_layers .= " :utf8" if $console_utf8;
+
+	my $in;
+	$in = $IN if ref($IN) eq "GLOB";
 	$in = \$IN if ref(\$IN) eq "GLOB";
-	open($in, '<', $console) unless defined($in);
+	open($in, "<$console_layers", $console) unless defined($in);
 	$in = \*STDIN unless defined($in);
 	$self->{IN} = $in;
-	my $out = $OUT if ref($OUT) eq "GLOB";
+
+	my $out;
+	$out = $OUT if ref($OUT) eq "GLOB";
 	$out = \$OUT if ref(\$OUT) eq "GLOB";
-	open($out, '>', $consoleOUT) unless defined($out);
+	open($out, ">$console_layers", $consoleOUT) unless defined($out);
 	$out = \*STDOUT unless defined($out);
 	$self->{OUT} = $out;
 
+	$self->{features}->{utf8} = grep(":utf8", PerlIO::get_layers($in));
+
 	return ($self->{IN}, $self->{OUT});
 }
 
+=head2 ornaments
+
+This is void implementation. Ornaments is B<not supported>.
+
+=cut
+sub ornaments
+{
+	return;
+}
+
+=head2 gethistory()
+
+B<GetHistory()>
+
+Returns copy of the history in Array.
+
+=cut
+sub gethistory
+{
+	my $self = shift;
+	return @{$self->{history}};
+}
+sub GetHistory
+{
+	return gethistory(@_);
+}
+
+=head2 sethistory($line1[, $line2[, ...]])
+
+B<SetHistory($line1[, $line2[, ...]])>
+
+rewrites all history by argument values.
+
+=cut
+sub sethistory
+{
+	my $self = shift;
+	if ($self->{features}->{utf8})
+	{
+		for (my $i = 0; $i < @_; $i++)
+		{
+			utf8::decode($_[$i]);
+		}
+	}
+	@{$self->{history}} = @_;
+	return 1;
+}
+sub SetHistory
+{
+	return sethistory(@_);
+}
+
+=head2 changehistory([$changehistory])
+
+If argument is specified, it allows to change history lines when argument value is true.
+
+Returns the old value.
+
+=cut
+sub changehistory
+{
+	my $self = shift;
+	my ($changehistory) = @_;
+	my $result = $self->{features}->{changehistory};
+	$self->{features}->{changehistory} = $changehistory if @_ >= 1;
+	return $result;
+}
+
 =head1 Other Methods and Functions
 
 =cut
+
 =head2 readkey([$echo])
 
 reads a key from input and echoes if I<echo> argument is C<TRUE>.
@@ -607,60 +730,6 @@ sub readkey
 	return $result;
 }
 
-=head2 minline([$minline])
-
-synonym of C<MinLine>.
-
-=cut
-sub minline
-{
-	return MinLine(@_);
-}
-
-=head2 changehistory([$changehistory])
-
-If argument is specified, it allows to change history lines when argument value is true.
-
-Returns the old value.
-
-=cut
-sub changehistory
-{
-	my $self = shift;
-	my ($changehistory) = @_;
-	my $result = $self->{features}->{changehistory};
-	$self->{features}->{changehistory} = $changehistory if @_ >= 1;
-	return $result;
-}
-
-=head2 history([$history])
-
-If argument is specified and ArrayRef, rewrites all history by argument elements.
-
-B<history([$line1[, $line2[, ...]]])>
-
-If first argument is not ArrayRef, rewrites all history by argument values.
-
-Returns copy of the old history in ArrayRef.
-
-=cut
-sub history
-{
-	my $self = shift;
-	my @result = @{$self->{history}};
-	if (@_ >= 1)
-	{
-		if (ref($_[0]) eq "ARRAY")
-		{
-			@{$self->{history}} = @{$_[0]};
-		} else
-		{
-			@{$self->{history}} = @_;
-		}
-	}
-	return \@result;
-}
-
 =head2 encode_controlchar($c)
 
 encodes if argument C<c> is a control character, otherwise returns argument C<c>.
@@ -694,9 +763,11 @@ sub encode_controlchar
 __END__
 =head1 UTF-8
 
-C<Term::ReadLine::Tiny> fully supports UTF-8.
+C<Term::ReadLine::Tiny> fully supports UTF-8, opens console input/output file handles with C<:utf8> layer by C<LANG>
+environment variable. You should set C<:utf8> layer explicitly, if input/output file handles specified with
+C<new()> or C<newTTY()>.
 
-	$term = Term::ReadLine::Tiny->new();
+	$term = Term::ReadLine::Tiny->new("", $in, $out);
 	binmode($term->IN, ":utf8");
 	binmode($term->OUT, ":utf8");
 	while ( defined($_ = $term->readline("Prompt: ")) )
